searchData={"items":[{"type":"behaviour","title":"CircuitBreakerSupervisor","doc":"A behaviour module and macro for implementing supervisors with a circuit\nbreaker for crashed children.","ref":"CircuitBreakerSupervisor.html"},{"type":"callback","title":"CircuitBreakerSupervisor.backoff/1","doc":"Given the number of attempts (how many times the process has crashed),\ndetermine how long to wait before restarting, in milliseconds. Returning\n`:timer.seconds(n)` also works.\n\nIf this callback is not implemented, all crashed processes will be restarted\nwithout a delay.","ref":"CircuitBreakerSupervisor.html#c:backoff/1"},{"type":"callback","title":"CircuitBreakerSupervisor.enabled?/1","doc":"Given the id of a supervised process, determine if it should be running.\n\nIf this callback is not implemented, all supervised processes will be\nenabled.\n\nThis callback should be used to connect monitored processes to feature flags.","ref":"CircuitBreakerSupervisor.html#c:enabled?/1"},{"type":"module","title":"CircuitBreakerSupervisor.Monitor","doc":"Monitors processes held by `CircuitBreakerSupervisor.Supervisor`.","ref":"CircuitBreakerSupervisor.Monitor.html"},{"type":"function","title":"CircuitBreakerSupervisor.Monitor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"CircuitBreakerSupervisor.Monitor.html#child_spec/1"},{"type":"function","title":"CircuitBreakerSupervisor.Monitor.start_link/1","doc":"","ref":"CircuitBreakerSupervisor.Monitor.html#start_link/1"},{"type":"module","title":"CircuitBreakerSupervisor.State","doc":"Finite state machine for tracking status of a single supervised process.\n\nAllowed states are:\n\n- running_disabled (feature flag turned off, should not be running)\n- running_in_startup (process is running, but hasn't been running long enough to be considered fixed)\n- running_past_startup (process is running, and we can reset the attempt counter because it has recovered)\n- stopped_disabled (feature flag turned off, should not be restarted)\n- stopped_in_backoff (process crashed, but we need to wait before retrying it)\n- stopped_past_backoff (process crashed and is ready to be retried, or hasn't been started)","ref":"CircuitBreakerSupervisor.State.html"},{"type":"function","title":"CircuitBreakerSupervisor.State.emit_start_telemetry/2","doc":"","ref":"CircuitBreakerSupervisor.State.html#emit_start_telemetry/2"},{"type":"function","title":"CircuitBreakerSupervisor.State.emit_stop_telemetry/2","doc":"","ref":"CircuitBreakerSupervisor.State.html#emit_stop_telemetry/2"},{"type":"function","title":"CircuitBreakerSupervisor.State.get_state/2","doc":"","ref":"CircuitBreakerSupervisor.State.html#get_state/2"},{"type":"function","title":"CircuitBreakerSupervisor.State.record_startup_crash/2","doc":"Increment the attempt_count to record a failed startup attempt.","ref":"CircuitBreakerSupervisor.State.html#record_startup_crash/2"},{"type":"module","title":"CircuitBreakerSupervisor.Supervisor","doc":"A regular supervisor that is started up to hold all the children supervised\nby the `CircuitBreakerSupervisor`.\n\n`Supervisor` is used rather than `DynamicSupervisor` because `Supervisor`\ntracks the ids that are assigned to children. `DynamicSupervisor` only returns\n`:undefined` ids when `which_children` is called, making it difficult to use.","ref":"CircuitBreakerSupervisor.Supervisor.html"},{"type":"function","title":"CircuitBreakerSupervisor.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"CircuitBreakerSupervisor.Supervisor.html#child_spec/1"},{"type":"function","title":"CircuitBreakerSupervisor.Supervisor.start_link/1","doc":"","ref":"CircuitBreakerSupervisor.Supervisor.html#start_link/1"},{"type":"extras","title":"Circuit Breaker Supervisor","doc":"# Circuit Breaker Supervisor","ref":"readme.html"},{"type":"extras","title":"Installation - Circuit Breaker Supervisor","doc":"Add `circuit_breaker_supervisor` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:circuit_breaker_supervisor, \"~> 0.0.0-rc.1\"}\n  ]\nend\n```\n\nDocs can be found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Circuit Breaker Supervisor","doc":"This package provides a macro that you can use to implement a custom supervisor:\n\n```elixir\ndefmodule MyApp.MySupervisor do\n  use CircuitBreakerSupervisor, poll_interval: :timer.seconds(1)\n\n  @impl true\n  def backoff(attempt) do\n    # example exponential backoff with jitter\n    trunc(:math.pow(attempt, 4) + 15 + :rand.uniform(30) * attempt)\n  end\n\n  @impl true\n  def enabled?(id) do\n    MyApp.FeatureFlagService.enabled?(id)\n  end\nend\n```\n\nThen start your custom supervisor within your application:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      # ...\n      {\n        CircuitBreakerSupervisor,\n        children: [\n          # children that should be managed by the circuit breaker, each one\n          # must have a unique id\n          %{\n            id: SomeService.Supervisor,\n            start: {SomeService.Supervisor, :start_link, [[]]},\n          },\n          %{\n            id: AnotherService.Supervisor,\n            start: {AnotherService.Supervisor, :start_link, [[]]},\n          }\n        ]\n      }\n      # start up other children outside of the circuit breaker...\n      {MyApp.MySupervisor, arg}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  # ...\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Telemetry - Circuit Breaker Supervisor","doc":"The following events are emitted when child processes are started / stopped:\n\n- `[:circuit_breaker_supervisor, :child, :stop]`\n- `[:circuit_breaker_supervisor, :child, :start]`","ref":"readme.html#telemetry"},{"type":"extras","title":"Measures - Circuit Breaker Supervisor","doc":"- `attempt_count` - The number of times the process has been restarted since the last time it was considered healthy.","ref":"readme.html#measures"},{"type":"extras","title":"Metadata - Circuit Breaker Supervisor","doc":"- `id` - The same id that is passed in with the `child_spec`","ref":"readme.html#metadata"},{"type":"extras","title":"Related Projects - Circuit Breaker Supervisor","doc":"- https://knock.app/blog/controlling-elixir-supervisors-at-runtime-with-feature-flags\n- https://github.com/mmzeeman/breaky","ref":"readme.html#related-projects"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}